use super::*;
use crate::search::youtube::VideoSearch;
use crate::spotify::PlayableKind;
use mockall::predicate;

#[tokio::test]
async fn should_search_track_by_spotify_id() {
    let spotify_id = "spotify:track:no-value-kek";
    let mut spotify_mock = spotify::MockSearchableClient::new();
    spotify_mock
        .expect_get_track()
        .with(predicate::eq("no-value-kek"))
        .returning(|_id| {
            Some(TrackInfo {
                name: "A name".to_string(),
                artists: vec!["Art1".to_string()],
                duration: Default::default(),
            })
        });
    let mut youtube_mock = youtube::MockSearchableClient::new();
    youtube_mock
        .expect_search_video()
        .returning(|_id, _sort_by| {
            Ok(VideoSearch {
                items: vec![Video {
                    title: "An example".to_string(),
                    video_id: "id123".to_string(),
                    author: "An Art".to_string(),
                    author_id: "artId123".to_string(),
                    author_url: "https://example.com".to_string(),
                    length_seconds: 42,
                    description: "A song".to_string(),
                    description_html: "A song 2".to_string(),
                    view_count: 0,
                    published: 0,
                    published_text: "".to_string(),
                    live_now: false,
                    paid: false,
                    premium: false,
                }],
            })
        });

    let engine = Engine::new_with_dependencies(Box::new(spotify_mock), Box::new(youtube_mock));
    let got = engine.get_song_from_spotify_id(spotify_id).await;

    assert_eq!(true, got.spotify_track.is_some());
    let boxed_st = Box::new(got.spotify_track.unwrap());
    assert_eq!(1, boxed_st.artists.len());
    assert_eq!("Art1".to_string(), boxed_st.artists.get(0).unwrap().clone());
    assert_eq!("A name".to_string(), boxed_st.name);

    assert_eq!(true, got.youtube_track.is_some());
    let boxed_yt = Box::new(got.youtube_track.unwrap());
    assert_eq!(1, boxed_yt.len());
    let got_video = boxed_yt.get(0).unwrap();
    assert_eq!("An example".to_string(), got_video.title);
}

#[tokio::test]
async fn should_search_album_by_spotify_id() {
    let spotify_id = "spotify:album:no-value-kek";
    let mut spotify_mock = spotify::MockSearchableClient::new();
    spotify_mock
        .expect_get_album()
        .with(predicate::eq("no-value-kek"))
        .returning(|_id| {
            Some(AlbumInfo {
                name: "An album".to_string(),
                artists: vec!["Art1".to_string(), "Art2".to_string()],
                genres: vec!["Rock".to_string(), "Hip-hop".to_string()],
                tracks: vec![TrackInfo {
                    name: "Track info 1".to_string(),
                    artists: vec!["Art1".to_string()],
                    duration: Default::default(),
                }],
            })
        });
    let youtube_mock = youtube::MockSearchableClient::new();

    let engine = Engine::new_with_dependencies(Box::new(spotify_mock), Box::new(youtube_mock));
    let got = engine.get_album_from_spotify_id(spotify_id).await;

    assert_eq!(true, got.spotify_album.is_some());
    let boxed_st = Box::new(got.spotify_album.unwrap());
    assert_eq!(2, boxed_st.artists.len());
    assert_eq!("Art1".to_string(), boxed_st.artists.get(0).unwrap().clone());
    assert_eq!("Art2".to_string(), boxed_st.artists.get(1).unwrap().clone());
    assert_eq!("An album".to_string(), boxed_st.name);
    assert_eq!(2, boxed_st.genres.len());
    assert_eq!("Rock".to_string(), boxed_st.genres.get(0).unwrap().clone());
    assert_eq!(
        "Hip-hop".to_string(),
        boxed_st.genres.get(1).unwrap().clone()
    );
    assert_eq!(1, boxed_st.tracks.len());
    assert_eq!(
        TrackInfo {
            name: "Track info 1".to_string(),
            artists: vec!["Art1".to_string()],
            duration: Default::default(),
        },
        boxed_st.tracks.get(0).unwrap().clone()
    );
}

#[tokio::test]
async fn should_search_playlist_by_spotify_id() {
    let spotify_id = "spotify:playlist:no-value-kek";
    let mut spotify_mock = spotify::MockSearchableClient::new();
    spotify_mock
        .expect_get_playlist()
        .with(predicate::eq("no-value-kek"))
        .returning(|_id| {
            Some(PlaylistInfo {
                name: "A playlist".to_string(),
                artists: vec!["Art1".to_string(), "Art2".to_string()],
                tracks: vec![PlayableKind::Track(TrackInfo {
                    name: "A track".to_string(),
                    artists: vec!["Art1".to_string()],
                    duration: Default::default(),
                })],
                owner: Some("Frodo".to_string()),
            })
        });
    let youtube_mock = youtube::MockSearchableClient::new();

    let engine = Engine::new_with_dependencies(Box::new(spotify_mock), Box::new(youtube_mock));
    let got = engine.get_playlist_from_spotify_id(spotify_id).await;

    assert_eq!(true, got.spotify_playlist.is_some());
    let boxed_st = Box::new(got.spotify_playlist.unwrap());
    assert_eq!(2, boxed_st.artists.len());
    assert_eq!("Art1".to_string(), boxed_st.artists.get(0).unwrap().clone());
    assert_eq!("Art2".to_string(), boxed_st.artists.get(1).unwrap().clone());
    assert_eq!("A playlist".to_string(), boxed_st.name);
    assert_eq!(1, boxed_st.tracks.len());
    assert_eq!(
        PlayableKind::Track(TrackInfo {
            name: "A track".to_string(),
            artists: vec!["Art1".to_string()],
            duration: Default::default(),
        }),
        boxed_st.tracks.get(0).unwrap().clone()
    );
    assert_eq!(Some("Frodo".to_string()), boxed_st.owner);
}
